@@ -14,6 +14,8 @@ import re
 import configparser
 from loguru import logger
 import hashlib
+import platform
+
 # from Crypto.Cipher import AES
 # from binascii import b2a_hex
 
@@ -21,14 +23,18 @@ import hashlib
 # sys.setdefaultencoding('utf8')
 # print("WARN:please configuration the configuraion_file variable")
 
-configuraion_file = "/data/apps/public/conf.ini"
-
+configuration_file = "/data/apps/public/conf.ini"
+configuration_path_prefix = "/data"
+system = platform.system()
+if system == "Darwin":
+    configuration_path_prefix = os.path.expanduser("~")
+configuration_file = "{}/apps/public/conf.ini".format(configuration_path_prefix)
 
 def getConfig():
-    print(configuraion_file)
+    print(configuration_file)
     import configparser
     config = configparser.ConfigParser()
-    config.read(configuraion_file)
+    config.read(configuration_file)
     return config
 
 
@@ -142,7 +148,7 @@ def getDatabase(section="mysql"):
         import configparser
         import pymysql
         config = configparser.ConfigParser()
-        config.read(configuraion_file)
+        config.read(configuration_file)
         host = config.get(section, "host")
         port = int(config.get(section, "port"))
         user = config.get(section, "user")
@@ -177,13 +183,16 @@ def query(database, sql, argumentTuple=(), timestamp2str=True, debug=True):
     if timestamp2str:
         timestamp_field_array = []
         for tp in cursor.description:
-            if tp[1] == pymysql.constants.FIELD_TYPE.TIMESTAMP:
+            if tp[1] == pymysql.constants.FIELD_TYPE.TIMESTAMP or tp[1] == pymysql.constants.FIELD_TYPE.DATETIME:
                 timestamp_field_array.append(tp[0])
 
         for row in cursor:
             for field in timestamp_field_array:
-                tmp_value = row[field].strftime("%Y-%m-%d %H:%M:%S")
-                row[field] = tmp_value
+                if row[field] is not None:
+                    tmp_value = row[field].strftime("%Y-%m-%d %H:%M:%S")
+                    row[field] = tmp_value
+                else:
+                    row[field] = ""
             rows.append(row)
     else:
         for row in cursor:
@@ -401,22 +410,36 @@ def dejson(content):
     """
     tmp_content = re.sub("\\s+", "", content)
     tmp_content = re.sub(":'", r':"', tmp_content)
+    tmp_content = re.sub("':", r'":', tmp_content)
     tmp_content = re.sub("',", r'",', tmp_content)
     tmp_content = re.sub(",'", r',"', tmp_content)
+    tmp_content = re.sub("[{]'([a-zA-Z]{2,})", r'{"\1', tmp_content)
 
     tmp_content = re.sub("\\['", r'["', tmp_content)
     tmp_content = re.sub("'}", r'"}', tmp_content)
     tmp_content = re.sub("']", r'"]', tmp_content)
 
-    tmp_content = re.sub("[{]([a-zA-Z])", r'{"\1', tmp_content)
+    # tmp_content = re.sub("[{]([a-zA-Z])", r'{"\1', tmp_content)
+    tmp_content = re.sub("""[{]([a-zA-Z])""", r'{"\1', tmp_content)
     tmp_content = re.sub("\\b([a-zA-Z])(\\w+):", r'\1\2":', tmp_content)
     tmp_content = re.sub(",([a-zA-Z])", r',"\1', tmp_content)
     return tmp_content
 
+# reconnecting mysql , 参考 https://www.kingname.info/2017/04/17/decorate-for-method/
+def pingmysql(original_function):
+    def wrapper(self, *args, **kwargs):
+        try:
+            self.database.ping()
+            result = original_function(self, *args, **kwargs)
+            return result
+        except Exception:
+            return 'an Exception raised.'
+    return wrapper
+
 
 class Mysql():
 
-    def __init__(self, configuraion_file="/data/apps/public/conf.ini", section="mysql"):
+    def __init__(self, configuraion_file=configuration_file, section="mysql"):
         self.database = None
         try:
             config = configparser.ConfigParser()
@@ -447,8 +470,8 @@ class Mysql():
             logger.error(e)
             traceback.print_exc()
 
+    @pingmysql
     def query(self, sql, argumentTuple=(), timestamp2str=True):
-        # self.database.ping()
         cursor = self.database.cursor()
         logger.debug("query sql:\t%s, arguments: %s" % (sql, argumentTuple))
         if len(argumentTuple) == 0:
@@ -460,21 +483,24 @@ class Mysql():
         if timestamp2str:
             timestamp_field_array = []
             for tp in cursor.description:
-                if tp[1] == pymysql.constants.FIELD_TYPE.TIMESTAMP:
+                if tp[1] == pymysql.constants.FIELD_TYPE.TIMESTAMP or tp[1] == pymysql.constants.FIELD_TYPE.DATETIME:
                     timestamp_field_array.append(tp[0])
 
             for row in cursor:
                 for field in timestamp_field_array:
-                    tmp_value = row[field].strftime("%Y-%m-%d %H:%M:%S")
-                    row[field] = tmp_value
+                    if row[field] is not None:
+                        tmp_value = row[field].strftime("%Y-%m-%d %H:%M:%S")
+                        row[field] = tmp_value
+                    else:
+                        row[field] = ""
                 rows.append(row)
         else:
             for row in cursor:
                 rows.append(row)
         return rows
 
+    @pingmysql
     def insert(self, tableName, dic, commit=True):
-        # self.database.ping()
         cursor = self.database.cursor()
         cols = []
         vals = []
@@ -498,8 +524,8 @@ class Mysql():
 
         return id
 
+    @pingmysql
     def update_by_id(self, tableName, id, dic, idFieldName="id", commit=True):
-        # self.database.ping()
         cursor = self.database.cursor()
         vals = []
         placeholders = []
@@ -513,14 +539,16 @@ class Mysql():
         update_sql = "update {0} set {1} where {2} = %s ".format(tableName, setting, idFieldName)
         logger.debug(update_sql)
 
+        result = None
         if commit:
             logger.debug(tuple(vals))
-            cursor.execute(update_sql, tuple(vals))
+            result = cursor.execute(update_sql, tuple(vals))
         if commit:
             self.database.commit()
+        return result
 
+    @pingmysql
     def execute(self, sql, argumentTuple=(), commit=True):
-        # self.database.ping()
         cursor = self.database.cursor()
         logger.debug("query sql:\t%s, arguments: %s" % (sql, argumentTuple))
         if len(argumentTuple) == 0:
@@ -530,10 +558,56 @@ class Mysql():
         if commit:
             self.database.commit()
 
+    @pingmysql
     def delete(self, sql, commit=True):
-        # self.database.ping()
         cursor = self.database.cursor()
         logger.debug("delete sql:\t" + sql)
         if commit:
             cursor.execute(sql)
             self.database.commit()
+
+    # Bulk Insert
+    @pingmysql
+    def bulk_insert(self, table_name, field_array, values_array, batch_size=100):
+        field_line = ",".join(field_array)
+        placeholder = ",".join(list(map(lambda x: "%s", field_array)))
+        logger.info("insert bulk data ")
+        ql = "INSERT INTO {} ({}) values ({})".format(table_name, field_line, placeholder)
+        logger.debug(ql)
+        cursor = self.database.cursor()
+        for start in range(0, len(values_array), batch_size):
+            logger.debug(values_array[start:start + batch_size])
+            cursor.executemany(ql, values_array[start:start + batch_size])
+            self.database.commit()
+
+    # Bulk Insert
+    @pingmysql
+    def bulk_insert2(self, table_name, data, batch_size=100):
+        """
+        insert the object array to the table use mysql batch insert commit api
+
+        :param table_name: table name
+        :param data:  object array
+        :param batch_size: size of batch to commit
+        :return: no return value
+        """
+        field_array = data[0].keys()
+        field_line = ",".join(field_array)
+        placeholder = ",".join(list(map(lambda x: "%s", field_array)))
+        logger.info("insert bulk data ")
+        ql = "INSERT INTO {} ({}) values ({})".format(table_name, field_line, placeholder)
+        logger.debug(ql)
+
+        values_array = []
+        for row in data:
+            tmp_ar = []
+            for i in field_array:
+                val = row.get(i)
+                tmp_ar.append(val)
+            values_array.append(tmp_ar)
+        print(values_array)
+        cursor = self.database.cursor()
+        for start in range(0, len(values_array), batch_size):
+            logger.debug(values_array[start:start + batch_size])
+            cursor.executemany(ql, values_array[start:start + batch_size])
+            self.database.commit()
\ No newline at end of file
